/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.tidalwave.openrdf.elmo;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.Arrays;
import org.openrdf.elmo.ElmoManager;
import org.openrdf.elmo.Entity;
import org.openrdf.elmo.exceptions.ElmoPersistException;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.util.As.NotFoundBehaviour;
import it.tidalwave.util.AsException;
import it.tidalwave.semantic.Wrapper;

/**
 *
 * @author fritz
 */
public class ElmoAsSupport
  {
    private static final String CLASS = ElmoAsSupport.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    /***************************************************************************
     *
     *
     **************************************************************************/
    private ElmoAsSupport()
      {
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public static <T> T as (@Nonnull final Object owner,
                            @Nonnull final Class<T> clazz)
      {
        return as(owner, clazz, new NotFoundBehaviour<T>()
          {
            public T run (@Nonnull final Throwable t)
              {
                throw new AsException(clazz, t);
              }
          });
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @CheckForNull
    public static <T> T as (@Nonnull final Object owner,
                            @Nonnull final Class<T> clazz,
                            @Nonnull final NotFoundBehaviour<T> notFoundBehaviour)
      {
        logger.fine("as(%s(%s), %s, %s)", owner, ((Entity)owner).getQName(), clazz, notFoundBehaviour);

        if (clazz.isAssignableFrom(owner.getClass()))
          {
            return (T)owner;
          }

        final Class<? extends T> implementation = (Class<? extends T>)Wrapper.findImplementation(clazz);
        final Class<? extends T> cl = (implementation != null) ? implementation : clazz;
        logger.finest(">>>> class: %s, implementationClass: %s", clazz, cl);


//        if (owner instanceof Thing)
//          {
//            logger.finer("Trying to answer by owl:sameAs...");
//            final Object as = ElmoOwlUtils.findSame((Thing)owner, cl);
//
//            if (as != null)
//              {
//                return (T)as;
//              }
//          }

        try
          {
            final ElmoManager em = ElmoManagerThreadLocal.get();
            Object as = em.designateEntity(owner, cl);

            if (!clazz.isAssignableFrom(as.getClass()))
              {
//                logger.fine("Designation failed for %s to %s: superclass: %s, interfaces: %s",
//                            owner, clazz, as.getClass().getSuperclass(), Arrays.toString(as.getClass().getInterfaces()));
//                logger.fine("Creating a new instance and using owl:sameAs");
//                final QName qName = new QName("urn:tidalwave:blank#" + new UUID().toString());
//                as = ElmoManagerThreadLocal.get().create(qName, cl);
//                ((Thing)as).getOwlSameAs().add((Thing)owner);
                throw new RuntimeException(String.format("Designation failed for %s to %s: superclass: %s, interfaces: %s",
                                                         owner, clazz, as.getClass().getSuperclass(), Arrays.toString(as.getClass().getInterfaces())));
              }

            return (T)as;
          }
        //FIXME: better to preventively probe clazz to check whether it's a concept
        catch (ElmoPersistException e) // concept not registered
          {
//            logger.severe(CLASS, "as()", e);
            logger.throwing(CLASS, "as()", e);
            return notFoundBehaviour.run(e);
//            throw new RuntimeException(e);
          }
      }
  }
