/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id: ElmoUtils.java,v 1c263a6d8ac9 2010/02/14 00:23:33 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.openrdf.elmo;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.xml.namespace.QName;
import com.eaio.uuid.UUID;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import org.openrdf.elmo.Entity;
import it.tidalwave.util.Displayable;
import it.tidalwave.netbeans.capabilitiesprovider.LookupFactory;
import it.tidalwave.openrdf.elmo.impl.ElmoAs;
import it.tidalwave.openrdf.elmo.impl.IdLazyLookup;
import it.tidalwave.openrdf.elmo.impl.AsLookup;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ElmoUtils.java,v 1c263a6d8ac9 2010/02/14 00:23:33 fabrizio $
 *
 ******************************************************************************/
public class ElmoUtils
  {
    private ElmoUtils()
      {
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public static String safeToString (@CheckForNull final Object object)
      {
        if (object == null)
          {
            return "null";
          }

        if (object.getClass().isArray())
          {
            String separator = "";
            final StringBuilder builder = new StringBuilder("[");

            for (final Object item : (Object[])object)
              {
                builder.append(separator);
                builder.append(safeToString(item));
                separator = ",";
              }

            builder.append("]");
            
            return builder.toString();
          }

        if (!object.getClass().isSynthetic())
          {
            return object.toString();
          }

        return String.format("%s@%x", object.getClass().getName(), System.identityHashCode(object));
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public static String toString (@Nonnull final Object object)
      {
        try
          {
            if (object instanceof Displayable)
              {
                return ((Displayable)object).getDisplayName();
              }

            return "";
          }
        catch (Exception e)
          {
            return "NOT INITIALIZED";
          }
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public static Lookup createLookup (@Nonnull final Object owner, @Nonnull final Object ... objects)
      {
        if (owner instanceof Entity)
          {
            return LookupFactory.createLookup(owner,
                                              // yes, owner is repeated, we want a copy of the owner
                                              Lookups.fixed(owner, new ElmoAs(owner)), // FIXME: use a Lazy Lookup
                                              new IdLazyLookup((Entity)owner),
                                              new AsLookup(owner),
                                              Lookups.fixed(objects));
          }

        return LookupFactory.createLookup(owner);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
//    @Nonnull
//    public static Lookup createSameAsLookup (@Nonnull final Thing thing)
//      {
//        return new OwlSameAsLazyLookup(thing);
//      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public static QName createId (@Nonnull final String nameSpace)
      {
        return new QName(nameSpace, (new UUID()).toString());
      }
  }
